import numpy as np
import pandas as pd

from .nlp import extract_words

from sklearn.feature_extraction.text import TfidfVectorizer, CountVectorizer
from sklearn.decomposition import PCA, NMF
from sklearn.cluster import KMeans, DBSCAN, MeanShift

import jieba
jieba.setLogLevel(60)

def pca(x, n_components=2):
    """
    Perform dimension reduction with the principal component analysis algorithm. The input data should be a pandas.Series of vectors.

    |
    """
    pca = PCA(n_components=n_components)
    return pd.Series(pca.fit_transform(list(x)).tolist(), index=x.index)

def nmf(x, n_components=2):
    """
    Perform dimension reduction with the non-negative matrix factorization algorithm. The input data should be a pandas.Series of vectors.

    |
    """
    nmf = NMF(n_components=n_components, init="random", random_state=0)
    return pd.Series(nmf.fit_transform(list(x)).tolist(), index=x.index)

def term_frequency(x, mode=0, max_features=None, return_feature_names=False):
    """
    Extract the words and vectorize each element in the pandas.Series by the frequency of each word.

    Args:
        x: The pandas.Series to be parsed.

        max_features: The maximum number of features

        return_feature_names: Return the token words or not.

        mode: 0: No single character words. The words may be overlapped.
              1: Have single character words. The words may be overlapped.
              2: No single character words. The words are not overlapped.
              3: Have single character words. The words are not overlapped.
              4: Only single characters.

    Returns:
        The vectorization result.

    |
    """

    if mode not in [0,1,2,3,4]:
        raise ValueError("The mode should be chosen from 0-4.")
    if not isinstance(x, pd.Series):
        raise ValueError("The type of the input variable should be pandas.Series.")

    vectorizer = CountVectorizer(max_features=max_features,
                                 lowercase=False,
                                 token_pattern="\S+")
    y = extract_words(x, isList=False, mode=mode, token=" ")
    y = pd.Series(vectorizer.fit_transform(y).toarray().tolist(), index=x.index)

    if return_feature_names:
        return (y, tf.get_feature_names())
    else:
        return y

def tfidf(x, mode=0, max_features=None, min_df=1, return_feature_names=False):
    """
    Extract the words and vectorize each element in the pandas.Series by the tfidf scores.

    Args:
        x: The pandas.Series to be parsed.

        max_features: The maximum number of features

        return_feature_names: Return the token words or not.

        mode: 0: No single character words. The words may be overlapped.
              1: Have single character words. The words may be overlapped.
              2: No single character words. The words are not overlapped.
              3: Have single character words. The words are not overlapped.
              4: Only single characters.

    Returns:
        The vectorization result.

    |
    """

    if mode not in [0,1,2,3,4]:
        raise ValueError("The mode should be chosen from 0-4.")
    if not isinstance(x, pd.Series):
        raise ValueError("The type of the input variable should be pandas.Series.")

    vectorizer = TfidfVectorizer(use_idf=True,
                                 max_features=max_features,
                                 min_df=min_df,
                                 token_pattern="\S+",
                                 lowercase=False,)
    y = extract_words(x, isList=False, mode=mode, token=" ")
    y = pd.Series(vectorizer.fit_transform(y).toarray().tolist(), index=y.index)

    if return_feature_names:
        return (y, tf.get_feature_names())
    else:
        return y



import chinese_converter

def to_simplified(s):
    res = list(chinese_converter.to_simplified(s))
    for i in range(len(s)):
        if s[i]=='泡' and res[i]!='泡':
            res[i]='泡'
    res = "".join(res)
    return res

def to_traditional(s):
    res = list(chinese_converter.to_traditional(s))
    res = "".join(res)
    return res



import re

import numpy as np
import pandas as pd

def _parse(func, x):
    if isinstance(x, str):
        return func(x)
    elif isinstance(x, pd.DataFrame):
        return x.applymap(func)
    else:
        raise ValueError("The type of the input variable should be string or pandas.DataFrame.")

def only_digits(x):
    """
    Only keeps the digits in a string or a pandas.DataFrame.

    Args:
        x: The content to be parsed. Either a string or a pandas.DataFrame.

    Returns:
        A new string or a pandas.DataFrame only includes digits.
    |
    """

    def func(_s):
        return "".join([x for x in re.findall(r'[0-9]', _s)])
    return _parse(func, x)

def only_zh(x):
    """
    Only keeps Chinese characters in a string or a pandas.DataFrame.

    Args:
        x: The content to be parsed. Either a string or a pandas.DataFrame.

    Returns:
        A new string or a pandas.DataFrame only includes Chinese characters.
    |
    """

    def func(_s):
        return "".join([x for x in re.findall(r'[\u4e00-\u9fff]+', _s)])
    return _parse(func, x)

def only_en(x):
    """
    Only keeps English alphabets in a string or a pandas.DataFrame.

    Args:
        x: The content to be parsed. Either a string or a pandas.DataFrame.

    Returns:
        A new string or a pandas.DataFrame only includes English alphabets.
    |
    """

    def func(_s):
        return re.sub(r'[^\x41-\x5A\x61-\x7A ]', '', _s)
    return _parse(func, x)

def remove_space(x):
    """
    Remove all the spaces in a string or a pandas.DataFrame.

    Args:
        x: The content to be parsed. Either a string or a pandas.DataFrame.

    Returns:
        A new string or a pandas.DataFrame without spaces.
    |
    """

    def func(_s):
        return "".join(_s.split())
    return _parse(func, x)

def remove_digits(x):
    """
    Remove all the digits in a string or a pandas.DataFrame.

    Args:
        x: The content to be parsed. Either a string or a pandas.DataFrame.

    Returns:
        A new string or a pandas.DataFrame without digits.
    |
    """

    def func(_s):
        return re.sub('[0-9]', '', _s)
    return _parse(func, x)

def remove_zh(x):
    """
    Remove all the Chinese characters in a string or a pandas.DataFrame.

    Args:
        x: The content to be parsed. Either a string or a pandas.DataFrame.

    Returns:
        A new string or a pandas.DataFrame without Chinese characters.
    |
    """

    def func(_s):
        return re.sub(r'[\u4e00-\u9fff]+', '', _s)
    return _parse(func, x)

def remove_en(x):
    """
    Remove all the English alphabets in a string or a pandas.DataFrame.

    Args:
        x: The content to be parsed. Either a string or a pandas.DataFrame.

    Returns:
        A new string or a pandas.DataFrame without English alphabets.
    |
    """

    def func(_s):
        return re.sub(r'[\x41-\x5A\x61-\x7A]', '', _s)
    return _parse(func, x)

def remove_punctuations(x):
    """
    Remove all the punctuations in a string or a pandas.DataFrame.

    Args:
        x: The content to be parsed. Either a string or a pandas.DataFrame.

    Returns:
        A new string or a pandas.DataFrame without punctuations.
    |
    """

    def func(_s):
        return re.sub(r'[^\w\s]','',_s)
    return _parse(func, x)

def fillna(x):
    """
    Fill the N/As in a pandas.DataFrame with an empty string.

    Args:
        x: A pandas.DataFrame content to be parsed.

    Returns:
        A pandas.DataFrame without N/As, which are substituted with empty strings.
    |
    """

    return x.fillna("")
#     return x.applymap(lambda a: a if pd.notnull(a) else "")

def toLower(x):
    """
    Transform alphabets to their lowercases.

    Args:
        x: The content to be parsed. Either a string or a pandas.DataFrame.

    Returns:
        A new string or a pandas.DataFrame where the alphabets are in lowercases.
    |
    """

    def func(_s):
        return _s.lower()
    return _parse(func, x)

def toUpper(x):
    """
    Transform alphabets to their uppercases.

    Args:
        x: The content to be parsed. Either a string or a pandas.DataFrame.

    Returns:
        A new string or a pandas.DataFrame where the alphabets are in uppercases.
    |
    """

    def func(_s):
        return _s.upper()
    return _parse(func, x)

def clean(x):
    """
    This function does the following:

    1. fillna(): Fill the N/As in a pandas.DataFrame with an empty string.

    2. toLower(): Transform alphabets to their lowercases.

    3. remove_punctuations(): Remove all the punctuations in a string or a pandas.DataFrame.

    4. remove_space(): Remove all the spaces in a string or a pandas.DataFrame.
    
    |
    """
    y = fillna(x)
    y = toLower(y)
    y = remove_punctuations(y)
    y = remove_space(y)
    return y
